@@ -1,6 +1,7 @@
 # pylint: disable=wrong-import-position
+# pylint: disable=too-many-lines
 #
-# Copyright (C) 2022 Mathieu Comandon <strider@strycore.com>
+# Copyright (C) 2009 Mathieu Comandon <mathieucomandon@gmail.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,44 +22,50 @@ import os
 import signal
 import sys
 import tempfile
-
 from datetime import datetime, timedelta
 from gettext import gettext as _
+from typing import List
 
 import gi
+
+from ..util.busy import BusyAsyncCall
+
 gi.require_version("Gdk", "3.0")
 gi.require_version("Gtk", "3.0")
 
-from gi.repository import Gio, GLib, Gtk, GObject
+from gi.repository import Gio, GLib, Gtk
 
-from lutris.runners import get_runner_names, import_runner, InvalidRunner, RunnerInstallationError
 from lutris import settings
-from lutris.api import parse_installer_url, get_runners
-from lutris.command import exec_command
+from lutris.api import get_runners, parse_installer_url
 from lutris.database import games as games_db
-from lutris.game import Game, export_game, import_game
-from lutris.installer import get_installers
-from lutris.gui.dialogs.download import simple_downloader
-from lutris.gui.dialogs import ErrorDialog, InstallOrPlayDialog, LutrisInitDialog
+from lutris.database.services import ServiceGameCollection
+from lutris.exception_backstops import init_exception_backstops
+from lutris.game import GAME_START, GAME_STOPPED, Game, export_game, import_game
+from lutris.gui.dialogs import ErrorDialog, InstallOrPlayDialog, NoticeDialog, display_error
+from lutris.gui.dialogs.delegates import CommandLineUIDelegate, InstallUIDelegate, LaunchUIDelegate
 from lutris.gui.dialogs.issue import IssueReportWindow
-from lutris.gui.installerwindow import InstallerWindow
+from lutris.gui.installerwindow import InstallationKind, InstallerWindow
 from lutris.gui.widgets.status_icon import LutrisStatusIcon
+from lutris.installer import get_installers
 from lutris.migrations import migrate
-from lutris.startup import init_lutris, run_all_checks, update_runtime
+from lutris.monitored_command import exec_command
+from lutris.runners import InvalidRunnerError, RunnerInstallationError, get_runner_names, import_runner
+from lutris.services import get_enabled_services
+from lutris.startup import init_lutris, run_all_checks
 from lutris.style_manager import StyleManager
 from lutris.util import datapath, log, system
 from lutris.util.http import HTTPError, Request
-from lutris.util.log import logger
+from lutris.util.log import file_handler, logger
+from lutris.util.savesync import save_check, show_save_stats, upload_save
 from lutris.util.steam.appmanifest import AppManifest, get_appmanifests
-from lutris.util.steam.config import get_steamapps_paths
-from lutris.services import get_enabled_services
-from lutris.database.services import ServiceGameCollection
+from lutris.util.steam.config import get_steamapps_dirs
 
 from .lutriswindow import LutrisWindow
 
+LUTRIS_EXPERIMENTAL_FEATURES_ENABLED = os.environ.get("LUTRIS_EXPERIMENTAL_FEATURES_ENABLED") == "1"
+
 
 class Application(Gtk.Application):
-
     def __init__(self):
         super().__init__(
             application_id="net.lutris.Lutris",
@@ -66,18 +73,23 @@ class Application(Gtk.Application):
             register_session=True,
         )
 
-        GObject.add_emission_hook(Game, "game-launch", self.on_game_launch)
-        GObject.add_emission_hook(Game, "game-start", self.on_game_start)
-        GObject.add_emission_hook(Game, "game-stop", self.on_game_stop)
-        GObject.add_emission_hook(Game, "game-install", self.on_game_install)
-        GObject.add_emission_hook(Game, "game-install-update", self.on_game_install_update)
-        GObject.add_emission_hook(Game, "game-install-dlc", self.on_game_install_dlc)
+        # Prepare the backstop logic just before the first emission hook (or connection) is
+        # established; this will apply to all connections from this point forward.
+        init_exception_backstops()
+
+        GAME_START.register(self.on_game_start)
+        GAME_STOPPED.register(self.on_game_stopped)
+        settings.SETTINGS_CHANGED.register(self.on_settings_changed)
 
         GLib.set_application_name(_("Lutris"))
+        GLib.set_prgname("net.lutris.Lutris")
+        self.force_updates = False
         self.css_provider = Gtk.CssProvider.new()
         self.window = None
+        self.launch_ui_delegate = LaunchUIDelegate()
+        self.install_ui_delegate = InstallUIDelegate()
 
-        self.running_games = Gio.ListStore.new(Game)
+        self._running_games = []
         self.app_windows = {}
         self.tray = None
 
@@ -85,7 +97,8 @@ class Application(Gtk.Application):
         self.style_manager = None
 
         if os.geteuid() == 0:
-            ErrorDialog(_("Running Lutris as root is not recommended and may cause unexpected issues"))
+            NoticeDialog(_("Do not run Lutris as root."))
+            sys.exit(2)
 
         try:
             self.css_provider.load_from_path(os.path.join(datapath.get(), "ui", "lutris.css"))
@@ -99,15 +112,17 @@ class Application(Gtk.Application):
 
     def add_arguments(self):
         if hasattr(self, "set_option_context_summary"):
-            self.set_option_context_summary(_(
-                "Run a game directly by adding the parameter lutris:rungame/game-identifier.\n"
-                "If several games share the same identifier you can use the numerical ID "
-                "(displayed when running lutris --list-games) and add "
-                "lutris:rungameid/numerical-id.\n"
-                "To install a game, add lutris:install/game-identifier."
-            ))
+            self.set_option_context_summary(
+                _(
+                    "Run a game directly by adding the parameter lutris:rungame/game-identifier.\n"
+                    "If several games share the same identifier you can use the numerical ID "
+                    "(displayed when running lutris --list-games) and add "
+                    "lutris:rungameid/numerical-id.\n"
+                    "To install a game, add lutris:install/game-identifier."
+                )
+            )
         else:
-            logger.warning("GLib.set_option_context_summary missing, " "was added in GLib 2.56 (Released 2018-03-12)")
+            logger.warning("GLib.set_option_context_summary missing, was added in GLib 2.56 (Released 2018-03-12)")
         self.add_main_option(
             "version",
             ord("v"),
@@ -132,6 +147,14 @@ class Application(Gtk.Application):
             _("Install a game from a yml file"),
             None,
         )
+        self.add_main_option(
+            "force",
+            ord("f"),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            _("Force updates"),
+            None,
+        )
         self.add_main_option(
             "output-script",
             ord("b"),
@@ -196,6 +219,22 @@ class Application(Gtk.Application):
             _("List all known Wine versions"),
             None,
         )
+        self.add_main_option(
+            "list-all-service-games",
+            ord("a"),
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.NONE,
+            _("List all games for all services in database"),
+            None,
+        )
+        self.add_main_option(
+            "list-service-games",
+            0,
+            GLib.OptionFlags.NONE,
+            GLib.OptionArg.STRING,
+            _("List all games for provided service in database"),
+            None,
+        )
         self.add_main_option(
             "install-runner",
             ord("r"),
@@ -228,6 +267,32 @@ class Application(Gtk.Application):
             _("Import a game"),
             None,
         )
+        if LUTRIS_EXPERIMENTAL_FEATURES_ENABLED:
+            self.add_main_option(
+                "save-stats",
+                0,
+                GLib.OptionFlags.NONE,
+                GLib.OptionArg.STRING,
+                _("Show statistics about a game saves"),
+                None,
+            )
+            self.add_main_option(
+                "save-upload",
+                0,
+                GLib.OptionFlags.NONE,
+                GLib.OptionArg.STRING,
+                _("Upload saves"),
+                None,
+            )
+            self.add_main_option(
+                "save-check",
+                0,
+                GLib.OptionFlags.NONE,
+                GLib.OptionArg.STRING,
+                _("Verify status of save syncing"),
+                None,
+            )
+
         self.add_main_option(
             "dest",
             0,
@@ -265,10 +330,11 @@ class Application(Gtk.Application):
     def do_startup(self):  # pylint: disable=arguments-differ
         """Sets up the application on first start."""
         Gtk.Application.do_startup(self)
+        file_handler.doRollover()
         signal.signal(signal.SIGINT, signal.SIG_DFL)
 
         action = Gio.SimpleAction.new("quit")
-        action.connect("activate", lambda *x: self.do_shutdown())
+        action.connect("activate", lambda *x: self.quit())
         self.add_action(action)
         self.add_accelerator("<Primary>q", "app.quit")
 
@@ -280,13 +346,11 @@ class Application(Gtk.Application):
             screen = self.window.props.screen  # pylint: disable=no-member
             Gtk.StyleContext.add_provider_for_screen(screen, self.css_provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
-    @staticmethod
-    def show_update_runtime_dialog():
+    def start_runtime_updates(self) -> None:
         if os.environ.get("LUTRIS_SKIP_INIT"):
             logger.debug("Skipping initialization")
         else:
-            init_dialog = LutrisInitDialog(update_runtime)
-            init_dialog.run()
+            self.window.start_runtime_updates(self.force_updates)
 
     def get_window_key(self, **kwargs):
         if kwargs.get("appid"):
@@ -296,14 +360,15 @@ class Application(Gtk.Application):
         if kwargs.get("installers"):
             return kwargs["installers"][0]["game_slug"]
         if kwargs.get("game"):
-            return str(kwargs["game"].id)
+            return kwargs["game"].id
         return str(kwargs)
 
-    def show_window(self, window_class, **kwargs):
+    def show_window(self, window_class, update_function=None, **kwargs):
         """Instantiate a window keeping 1 instance max
 
         Params:
             window_class (Gtk.Window): class to create the instance from
+            update_function (Callable): Function to initialize or update the window (if possible before being shown)
             kwargs (dict): Additional arguments to pass to the instanciated window
 
         Returns:
@@ -312,7 +377,10 @@ class Application(Gtk.Application):
         window_key = str(window_class.__name__) + self.get_window_key(**kwargs)
         if self.app_windows.get(window_key):
             self.app_windows[window_key].present()
-            return self.app_windows[window_key]
+            window_inst = self.app_windows[window_key]
+            if update_function:
+                update_function(window_inst)
+            return window_inst
         if issubclass(window_class, Gtk.Dialog):
             if "parent" in kwargs:
                 window_inst = window_class(**kwargs)
@@ -322,26 +390,33 @@ class Application(Gtk.Application):
         else:
             window_inst = window_class(application=self, **kwargs)
         window_inst.connect("destroy", self.on_app_window_destroyed, self.get_window_key(**kwargs))
+        if update_function:
+            update_function(window_inst)
         self.app_windows[window_key] = window_inst
-        logger.debug("Showing window %s", window_key)
         window_inst.show()
         return window_inst
 
-    def show_installer_window(self, installers, service=None, appid=None, is_update=False):
+    def show_installer_window(self, installers, service=None, appid=None, installation_kind=InstallationKind.INSTALL):
         self.show_window(
-            InstallerWindow,
-            installers=installers,
-            service=service,
-            appid=appid,
-            is_update=is_update
+            InstallerWindow, installers=installers, service=service, appid=appid, installation_kind=installation_kind
         )
 
+    def show_lutris_installer_window(self, game_slug):
+        def on_installers_ready(installers, error):
+            if error:
+                display_error(error, parent=self.window)
+            elif installers:
+                self.show_installer_window(installers)
+            else:
+                ErrorDialog(_("No installer available."), parent=self.window)
+
+        BusyAsyncCall(get_installers, on_installers_ready, game_slug=game_slug)
+
     def on_app_window_destroyed(self, app_window, window_key):
         """Remove the reference to the window when it has been destroyed"""
         window_key = str(app_window.__class__.__name__) + window_key
         try:
             del self.app_windows[window_key]
-            logger.debug("Removed window %s", window_key)
         except KeyError:
             logger.warning("Failed to remove window %s", window_key)
             logger.info("Available windows: %s", ", ".join(self.app_windows.keys()))
@@ -356,39 +431,16 @@ class Application(Gtk.Application):
         """Output a script to a file.
         The script is capable of launching a game without the client
         """
+
+        def on_error(error: BaseException) -> None:
+            logger.exception("Unable to generate script: %s", error)
+
         game = Game(db_game["id"])
-        game.load_config()
-        game.write_script(script_path)
-
-    def do_command_line(self, command_line):  # noqa: C901  # pylint: disable=arguments-differ
-        # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches
-        # pylint: disable=too-many-statements
-        # TODO: split into multiple methods to reduce complexity (35)
-        options = command_line.get_options_dict()
-
-        # Use stdout to output logs, only if no command line argument is
-        # provided
-        argc = len(sys.argv) - 1
-        if "-d" in sys.argv or "--debug" in sys.argv:
-            argc -= 1
-        if not argc:
-            # Switch back the log output to stderr (the default in Python)
-            # to avoid messing with any output from command line options.
-
-            # Use when targeting Python 3.7 minimum
-            # console_handler.setStream(sys.stderr)
-
-            # Until then...
-            logger.removeHandler(log.console_handler)
-            log.console_handler = logging.StreamHandler(stream=sys.stdout)
-            log.console_handler.setFormatter(log.SIMPLE_FORMATTER)
-            logger.addHandler(log.console_handler)
-
-        # Set up logger
-        if options.contains("debug"):
-            log.console_handler.setFormatter(log.DEBUG_FORMATTER)
-            logger.setLevel(logging.DEBUG)
+        game.game_error.register(on_error)
+        game.reload_config()
+        game.write_script(script_path, self.launch_ui_delegate)
 
+    def do_handle_local_options(self, options):
         # Text only commands
 
         # Print Lutris version and exit
@@ -397,11 +449,37 @@ class Application(Gtk.Application):
             print(executable_name + "-" + settings.VERSION)
             logger.setLevel(logging.NOTSET)
             return 0
+        return -1  # continue command line processes
 
+    def do_command_line(self, command_line):  # noqa: C901  # pylint: disable=arguments-differ
+        # pylint: disable=too-many-locals,too-many-return-statements,too-many-branches
+        # pylint: disable=too-many-statements
+        # TODO: split into multiple methods to reduce complexity (35)
+        options = command_line.get_options_dict()
+
+        # Use stdout to output logs, only if no command line argument is
+        # provided.
+        argc = len(sys.argv) - 1
+        if "-d" in sys.argv or "--debug" in sys.argv:
+            argc -= 1
+        if not argc:
+            # Switch back the log output to stderr (the default in Python)
+            # to avoid messing with any output from command line options.
+            log.console_handler.setStream(sys.stderr)
+
+        # Set up logger
+        if options.contains("debug"):
+            log.console_handler.setFormatter(log.DEBUG_FORMATTER)
+            logger.setLevel(logging.DEBUG)
+
+        if options.contains("force"):
+            self.force_updates = True
+
+        logger.info("Starting Lutris %s", settings.VERSION)
         init_lutris()
         migrate()
-        run_all_checks()
 
+        run_all_checks()
         if options.contains("dest"):
             dest_dir = options.lookup_value("dest").get_string()
         else:
@@ -409,15 +487,46 @@ class Application(Gtk.Application):
 
         # List game
         if options.contains("list-games"):
-            game_list = games_db.get_games(filters=(
-                {"installed": 1} if options.contains("installed") else None)
-            )
+            game_list = games_db.get_games(filters=({"installed": 1} if options.contains("installed") else None))
             if options.contains("json"):
                 self.print_game_json(command_line, game_list)
             else:
                 self.print_game_list(command_line, game_list)
             return 0
 
+        # List specified service games
+        if options.contains("list-service-games"):
+            service = options.lookup_value("list-service-games").get_string()
+            game_list = games_db.get_games(filters={"installed": 1, "service": service})
+            service_game_list = ServiceGameCollection.get_for_service(service)
+            for game in service_game_list:
+                game["installed"] = any(("service_id", game["appid"]) in item.items() for item in game_list)
+            if options.contains("installed"):
+                service_game_list = [d for d in service_game_list if d["installed"]]
+            if options.contains("json"):
+                self.print_service_game_json(command_line, service_game_list)
+            else:
+                self.print_service_game_list(command_line, service_game_list)
+            return 0
+
+        # List all service games
+        if options.contains("list-all-service-games"):
+            game_list = games_db.get_games(filters={"installed": 1})
+            service_game_list = ServiceGameCollection.get_service_games()
+            for game in service_game_list:
+                game["installed"] = any(
+                    ("service_id", game["appid"]) in item.items()
+                    for item in game_list
+                    if item["service"] == game["service"]
+                )
+            if options.contains("installed"):
+                service_game_list = [d for d in service_game_list if d["installed"]]
+            if options.contains("json"):
+                self.print_service_game_json(command_line, service_game_list)
+            else:
+                self.print_service_game_list(command_line, service_game_list)
+            return 0
+
         # List Steam games
         if options.contains("list-steam-games"):
             self.print_steam_list(command_line)
@@ -434,7 +543,7 @@ class Application(Gtk.Application):
             return 0
 
         # List Wine Runners
-        if options.contains("list-wine-runners"):
+        if options.contains("list-wine-versions"):
             self.print_wine_runners()
             return 0
 
@@ -466,6 +575,41 @@ class Application(Gtk.Application):
                 import_game(filepath, dest_dir)
             return 0
 
+        if LUTRIS_EXPERIMENTAL_FEATURES_ENABLED:
+
+            def get_game_match(slug):
+                # First look for an exact match
+                games = games_db.get_games_by_slug(slug)
+                if not games:
+                    # Then look for matches
+                    games = games_db.get_games(searches={"slug": slug})
+                if len(games) > 1:
+                    self._print(
+                        command_line,
+                        "Multiple games matching %s: %s" % (slug, ",".join(game["slug"] for game in games)),
+                    )
+                    return
+                if not games:
+                    self._print(command_line, "No matching game for %s" % slug)
+                    return
+                return Game(games[0]["id"])
+
+            if options.contains("save-stats"):
+                game = get_game_match(options.lookup_value("save-stats").get_string())
+                if game:
+                    show_save_stats(game, output_format="json" if options.contains("json") else "text")
+                return 0
+            if options.contains("save-upload"):
+                game = get_game_match(options.lookup_value("save-upload").get_string())
+                if game:
+                    upload_save(game)
+                return 0
+            if options.contains("save-check"):
+                game = get_game_match(options.lookup_value("save-check").get_string())
+                if game:
+                    save_check(game)
+                return 0
+
         # Execute command in Lutris context
         if options.contains("exec"):
             command = options.lookup_value("exec").get_string()
@@ -476,8 +620,8 @@ class Application(Gtk.Application):
             IssueReportWindow(application=self)
             return 0
 
+        url = options.lookup_value(GLib.OPTION_REMAINING)
         try:
-            url = options.lookup_value(GLib.OPTION_REMAINING)
             installer_info = self.get_lutris_action(url)
         except ValueError:
             self._print(command_line, _("%s is not a valid URI") % url.get_strv())
@@ -487,6 +631,9 @@ class Application(Gtk.Application):
         action = installer_info["action"]
         service = installer_info["service"]
         appid = installer_info["appid"]
+        launch_config_name = installer_info["launch_config_name"]
+
+        self.launch_ui_delegate = CommandLineUIDelegate(launch_config_name)
 
         if options.contains("output-script"):
             action = "write-script"
@@ -508,9 +655,10 @@ class Application(Gtk.Application):
                 except (KeyError, IndexError):
                     file_name = os.path.basename(installer_file)
                 file_path = os.path.join(tempfile.gettempdir(), file_name)
-                self._print(command_line, _("download {url} to {file} started").format(
-                    url=installer_file, file=file_path))
-                with open(file_path, 'wb') as dest_file:
+                self._print(
+                    command_line, _("download {url} to {file} started").format(url=installer_file, file=file_path)
+                )
+                with open(file_path, "wb") as dest_file:
                     dest_file.write(request.content)
                 installer_file = file_path
                 action = "install"
@@ -535,8 +683,9 @@ class Application(Gtk.Application):
             elif action == "install":
                 # Installers can use game or installer slugs
                 self.quit_on_game_exit = True
-                db_game = games_db.get_game_by_field(game_slug, "slug") \
-                    or games_db.get_game_by_field(game_slug, "installer_slug")
+                db_game = games_db.get_game_by_field(game_slug, "slug") or games_db.get_game_by_field(
+                    game_slug, "installer_slug"
+                )
             else:
                 # Dazed and confused, try anything that might works
                 db_game = (
@@ -564,8 +713,8 @@ class Application(Gtk.Application):
             if db_game and db_game["installed"]:
                 # Game found but no action provided, ask what to do
                 dlg = InstallOrPlayDialog(db_game["name"])
-                if not dlg.action_confirmed:
-                    action = None
+                if not dlg.action:
+                    action = "cancel"
                 elif dlg.action == "play":
                     action = "rungame"
                 elif dlg.action == "install":
@@ -582,6 +731,11 @@ class Application(Gtk.Application):
                 service.install(service_game)
                 return 0
 
+        if action == "cancel":
+            if not self.window.is_visible():
+                self.quit()
+            return 0
+
         if action == "install":
             installers = get_installers(
                 game_slug=game_slug,
@@ -590,117 +744,106 @@ class Application(Gtk.Application):
             )
             if installers:
                 self.show_installer_window(installers)
-
         elif action in ("rungame", "rungameid"):
             if not db_game or not db_game["id"]:
                 logger.warning("No game found in library")
                 if not self.window.is_visible():
-                    self.do_shutdown()
+                    self.quit()
                 return 0
+
+            def on_error(error: BaseException) -> None:
+                logger.exception("Unable to launch game: %s", error)
+
             game = Game(db_game["id"])
-            self.on_game_launch(game)
-        else:
-            Application.show_update_runtime_dialog()
+            game.game_error.register(on_error)
+            game.launch(self.launch_ui_delegate)
+
+            if game.state == game.STATE_STOPPED and not self.window.is_visible():
+                self.quit()
+        elif self.window:
+            # If we're showing the window, it will handle the delegated UI
+            # from here on out, no matter what command line we got.
+            self.launch_ui_delegate = self.window
+            self.install_ui_delegate = self.window
             self.window.present()
+            self.start_runtime_updates()
             # If the Lutris GUI is started by itself, don't quit it when a game stops
             self.quit_on_game_exit = False
         return 0
 
-    def on_game_launch(self, game):
-        game.launch()
-        return True  # Return True to continue handling the emission hook
+    def on_settings_changed(self, setting_key, new_value):
+        if setting_key == "preferred_theme":
+            self.style_manager.preferred_theme = new_value
+        elif setting_key == "show_tray_icon" and self.window:
+            if self.window.get_visible():
+                self.set_tray_icon()
+        return True
 
-    def on_game_start(self, game):
-        self.running_games.append(game)
-        if settings.read_setting("hide_client_on_game_start") == "True":
+    def on_game_start(self, game: Game) -> None:
+        self._running_games.append(game)
+        if self.window and settings.read_bool_setting("hide_client_on_game_start"):
             self.window.hide()  # Hide launcher window
-        return True
 
-    def on_game_install(self, game):
-        """Request installation of a game"""
-        if game.service and game.service != "lutris":
-            service = get_enabled_services()[game.service]()
-            db_game = ServiceGameCollection.get_game(service.id, game.appid)
-            if not db_game:
-                logger.error("Can't find %s for %s", game.name, service.name)
-                return True
+    def on_game_stopped(self, game: Game) -> None:
+        """Callback to quit Lutris is last game stops while the window is hidden."""
+        running_game_ids = [g.id for g in self._running_games]
+        if game.id in running_game_ids:
+            logger.debug("Removing %s from running IDs", game.id)
             try:
-                game_id = service.install(db_game)
-            except ValueError as e:
-                logger.debug(e)
-                game_id = None
-
-            if game_id:
-                game = Game(game_id)
-                game.launch()
-            return True
-        if not game.slug:
-            raise ValueError("Invalid game passed: %s" % game)
-            # return True
-        installers = get_installers(game_slug=game.slug)
-        if installers:
-            self.show_installer_window(installers)
-        else:
-            ErrorDialog(_("There is no installer available for %s.") % game.name, parent=self.window)
-        return True
-
-    def on_game_install_update(self, game):
-        service = get_enabled_services()[game.service]()
-        db_game = games_db.get_game_by_field(game.id, "id")
-        installers = service.get_update_installers(db_game)
-        if installers:
-            self.show_installer_window(installers, service, game.appid, is_update=True)
-        else:
-            ErrorDialog(_("No updates found"))
-        return True
-
-    def on_game_install_dlc(self, game):
-        service = get_enabled_services()[game.service]()
-        db_game = games_db.get_game_by_field(game.id, "id")
-        installers = service.get_dlc_installers_runner(db_game, db_game["runner"])
-        if installers:
-            self.show_installer_window(installers, service, game.appid)
-        else:
-            ErrorDialog(_("No DLC found"))
-        return True
-
-    def get_running_game_ids(self):
-        ids = []
-        for i in range(self.running_games.get_n_items()):
-            game = self.running_games.get_item(i)
-            ids.append(str(game.id))
-        return ids
-
-    def get_game_by_id(self, game_id):
-        for i in range(self.running_games.get_n_items()):
-            game = self.running_games.get_item(i)
-            if str(game.id) == str(game_id):
-                return game
-        return None
-
-    def on_game_stop(self, game):
-        """Callback to remove the game from the running games"""
-        ids = self.get_running_game_ids()
-        if str(game.id) in ids:
-            try:
-                self.running_games.remove(ids.index(str(game.id)))
+                del self._running_games[running_game_ids.index(game.id)]
             except ValueError:
                 pass
+        elif running_game_ids:
+            logger.warning("%s not in %s", game.id, running_game_ids)
         else:
-            logger.warning("%s not in %s", game.id, ids)
+            logger.debug("Game has already been removed from running IDs?")
 
-        game.emit("game-stopped")
-        if settings.read_setting("hide_client_on_game_start") == "True" and not self.quit_on_game_exit:
+        if self.window and settings.read_bool_setting("hide_client_on_game_start") and not self.quit_on_game_exit:
             self.window.show()  # Show launcher window
-        elif not self.window.is_visible():
-            if self.running_games.get_n_items() == 0:
+        elif not self.window or not self.window.is_visible():
+            if not self.has_running_games:
                 if self.quit_on_game_exit or not self.has_tray_icon():
-                    self.do_shutdown()
-        return True
+                    self.quit()
+
+    def get_launch_ui_delegate(self):
+        return self.launch_ui_delegate
+
+    def get_running_games(self) -> List[Game]:
+        # This method reflects games that have stopped even if the 'game-stopped' signal
+        # has not been handled yet; that handler will still clean up the list though.
+        return [g for g in self._running_games if g.state != g.STATE_STOPPED]
+
+    @property
+    def has_running_games(self):
+        return bool(self.get_running_games())
+
+    def get_running_game_ids(self) -> List[str]:
+        """Returns the ids of the games presently running."""
+        return [game.id for game in self.get_running_games()]
+
+    def is_game_running_by_id(self, game_id: str) -> bool:
+        """True if the ID is the ID of a game that is running."""
+        return bool(game_id and str(game_id) in self.get_running_game_ids())
+
+    def get_game_by_id(self, game_id: str) -> Game:
+        """Returns the game with the ID given; if it's running this is the running
+        game instance, and if not it's a fresh copy off the database."""
+        for game in self.get_running_games():
+            if game.id == str(game_id):
+                return game
+
+        return Game(game_id)
 
     @staticmethod
     def get_lutris_action(url):
-        installer_info = {"game_slug": None, "revision": None, "action": None, "service": None, "appid": None}
+        installer_info = {
+            "game_slug": None,
+            "revision": None,
+            "action": None,
+            "service": None,
+            "appid": None,
+            "launch_config_name": None,
+        }
 
         if url:
             url = url.get_strv()
@@ -708,8 +851,6 @@ class Application(Gtk.Application):
         if url:
             url = url[0]
             installer_info = parse_installer_url(url)
-            if installer_info is False:
-                raise ValueError
         return installer_info
 
     def print_game_list(self, command_line, game_list):
@@ -735,21 +876,43 @@ class Application(Gtk.Application):
                 "platform": game["platform"] or None,
                 "year": game["year"] or None,
                 "directory": game["directory"] or None,
-                "hidden": bool(game["hidden"]),
-                "playtime": (
-                    str(timedelta(hours=game["playtime"]))
-                    if game["playtime"] else None
+                "playtime": (str(timedelta(hours=game["playtime"])) if game["playtime"] else None),
+                "lastplayed": (str(datetime.fromtimestamp(game["lastplayed"])) if game["lastplayed"] else None),
+            }
+            for game in game_list
+        ]
+        self._print(command_line, json.dumps(games, indent=2))
+
+    def print_service_game_list(self, command_line, game_list):
+        for game in game_list:
+            self._print(
+                command_line,
+                "{:4} | {:<40} | {:<40} | {:<15} | {:<15}".format(
+                    game["id"],
+                    game["name"][:40],
+                    game["slug"][:40],
+                    game["service"][:15],
+                    "true" if game["installed"] else "false"[:15],
                 ),
-                "lastplayed": (
-                    str(datetime.fromtimestamp(game["lastplayed"]))
-                    if game["lastplayed"] else None
-                )
-            } for game in game_list
+            )
+
+    def print_service_game_json(self, command_line, game_list):
+        games = [
+            {
+                "id": game["id"],
+                "slug": game["slug"],
+                "name": game["name"],
+                "service": game["service"],
+                "appid": game["appid"],
+                "installed": game["installed"],
+                "details": game["details"],
+            }
+            for game in game_list
         ]
         self._print(command_line, json.dumps(games, indent=2))
 
     def print_steam_list(self, command_line):
-        steamapps_paths = get_steamapps_paths()
+        steamapps_paths = get_steamapps_dirs()
         for path in steamapps_paths if steamapps_paths else []:
             appmanifest_files = get_appmanifests(path)
             for appmanifest_file in appmanifest_files:
@@ -776,15 +939,15 @@ class Application(Gtk.Application):
             monitored_command.stop()
 
     def print_steam_folders(self, command_line):
-        steamapps_paths = get_steamapps_paths()
+        steamapps_paths = get_steamapps_dirs()
         if steamapps_paths:
             for path in steamapps_paths:
                 self._print(command_line, path)
 
     def print_runners(self):
-        runnersName = get_runner_names()
-        sortednames = sorted(runnersName.keys(), key=lambda x: x.lower())
-        for name in sortednames:
+        runner_names = get_runner_names()
+        sorted_names = sorted(runner_names, key=lambda x: x.lower())
+        for name in sorted_names:
             print(name)
 
     def print_wine_runners(self):
@@ -822,12 +985,11 @@ class Application(Gtk.Application):
         if os.path.isdir(runner_path):
             print(f"Wine version '{version}' is already installed.")
         else:
-
             try:
                 runner = import_runner("wine")
-                runner().install(downloader=simple_downloader, version=version)
+                runner().install(self.install_ui_delegate, version=version)
                 print(f"Wine version '{version}' has been installed.")
-            except (InvalidRunner, RunnerInstallationError) as ex:
+            except (InvalidRunnerError, RunnerInstallationError) as ex:
                 print(ex.message)
 
     def wine_runner_uninstall(self, version):
@@ -838,27 +1000,28 @@ class Application(Gtk.Application):
             system.remove_folder(runner_path)
             print(f"Wine version '{version}' has been removed.")
         else:
-            print(f"""
+            print(
+                f"""
 Specified version of Wine is not installed: {version}.
-Please check if the Wine Runner and specified version are installed (for that use --list-wine-runners).
+Please check if the Wine Runner and specified version are installed (for that use --list-wine-versions).
 Also, check that the version specified is in the correct format.
-                """)
+                """
+            )
 
     def install_cli(self, runner_name):
         """
         install the runner provided in prepare_runner_cli()
         """
 
-        runner_path = os.path.join(settings.RUNNER_DIR, runner_name)
-        if os.path.isdir(runner_path):
-            print(f"'{runner_name}' is already installed.")
-        else:
-            try:
-                runner = import_runner(runner_name)
-                runner().install(version=None, downloader=simple_downloader, callback=None)
+        try:
+            runner = import_runner(runner_name)()
+            if runner.is_installed():
+                print(f"'{runner_name}' is already installed.")
+            else:
+                runner.install(self.install_ui_delegate, version=None, callback=None)
                 print(f"'{runner_name}' has been installed")
-            except (InvalidRunner, RunnerInstallationError) as ex:
-                print(ex.message)
+        except (InvalidRunnerError, RunnerInstallationError) as ex:
+            print(ex.message)
 
     def uninstall_runner_cli(self, runner_name):
         """
@@ -868,7 +1031,7 @@ Also, check that the version specified is in the correct format.
         try:
             runner_class = import_runner(runner_name)
             runner = runner_class()
-        except InvalidRunner:
+        except InvalidRunnerError:
             logger.error("Failed to import Runner: %s", runner_name)
             return
         if not runner.is_installed():
@@ -883,7 +1046,8 @@ Also, check that the version specified is in the correct format.
     def do_shutdown(self):  # pylint: disable=arguments-differ
         logger.info("Shutting down Lutris")
         if self.window:
-            settings.write_setting("selected_category", self.window.selected_category)
+            selected_category = "%s:%s" % self.window.selected_category
+            settings.write_setting("selected_category", selected_category)
             self.window.destroy()
         Gtk.Application.do_shutdown(self)
 
